# 
# 
# PointPickViewer
# 
# Copyright (C) SAXS Group, Photon Factory, KEK, 2017 - 
# 
# 
# -- PointPickViewer uses the following system
# 
#    - Python 3.9 (https://www.python.org/)
#      Copyright (C) 2001-2020 Python Software Foundation; All Rights Reserved
# 
# 


import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mat
from matplotlib.colors import Normalize
from matplotlib import gridspec


xdata = np.zeros(1)
ydata = np.zeros(1)
zdata = np.zeros(1)
count=0

min_value = 0.0
max_value = 0.0

window_title = ''
plot_title = ''
x_title = ''
y_title = ''
z_title = ''

label_std = np.array([500, 200, 100, 50, 20, 10, 5, 1])










def Set_xdata(x):
	global xdata
	xdata = np.array(x)
	return xdata.size

def Set_ydata(y):
	global ydata
	ydata = np.array(y)
	return ydata.size

def Set_zdata(z):
	global zdata
	global count
	if count==0:
		zdata = np.array(z)
	else:
		zdata = np.append(zdata, z)
	
	count += 1
	return count
	
	
def ResetCount():
	global count
	
	count = 0
	return count


def GetLabelStd(sub):
	global label_std
	
	ret = 0
	for num in label_std:
		div = sub // num
		
		if (sub > num) and ((div > 2) and (div <= 15)):
			ret = num
			break
		if ret==0:
			ret = 1
	return ret

def GetYlabel(min, max):
	sub = max - min
	std = GetLabelStd(sub)
	ret = np.array([min])
	
	start = min // std
	last = max // std
	
	for i in range(start, last):
		add = (i + 1) * std
		if (add > min) and (add < max):
			ret = np.append(ret, add)
	if(ret.max() != max):
		ret = np.append(ret, max)
	return ret

def PlotOne(imagepath, cmap_str):
	global xdata
	global zdata
	global window_title
	global plot_title
	global x_title
	global z_title
	global min_value
	global max_value
	
	fig = plt.figure(figsize = (12, 6))
	ax = fig.add_subplot(111)
	fig.canvas.set_window_title(window_title)
	ax.set_title(plot_title)
	ax.set_xlabel(x_title)
	ax.set_ylabel(z_title)

	colormap = plt.get_cmap(cmap_str)
	norm = Normalize(vmin=min_value, vmax=max_value)
	
	c = colormap(norm(zdata[0]))
	
	for j in range(len(zdata[0]) - 1):
		ax.plot(xdata[j:j+2], zdata[0][j:j+2], color=c[j+1])
	plt.grid(True)
	fig.subplots_adjust(left=0.125)
	
	if len(imagepath) > 0:
		plt.savefig(imagepath)
	plt.show()
	
def Plot3D(plotmode, imagepath, cmap_str):
	global xdata
	global ydata
	global zdata
	global count
	global window_title
	global plot_title
	global x_title
	global y_title
	global z_title
	global min_value
	global max_value
	
	if len(ydata) > 15:
		ylabel = GetYlabel(ydata.min(), ydata.max())
	else:
		ylabel = ydata
	
	
	
	XX,YY = np.meshgrid(xdata, ydata)
	
	fig = plt.figure(figsize=(12, 6))
	fig.canvas.set_window_title(window_title)
	
	if plotmode==3:
		spec = gridspec.GridSpec(ncols=2, nrows=1, width_ratios=[1.3, 1])
		ax1 = fig.add_subplot(spec[0], projection='3d')
		surf = ax1.plot_surface(XX, YY, zdata, cmap=cmap_str, norm=Normalize(vmin=min_value, vmax=max_value))
		cb = fig.colorbar(surf, ax=ax1)
		
		ax1.set_title(plot_title)
		ax1.set_xlabel(x_title)
		ax1.set_ylabel(y_title)
		ax1.set_zlabel(z_title)
		
		ax1.set_yticks(ylabel)
		
		ax2 = fig.add_subplot(spec[1], projection='3d')
		ax2.plot_wireframe(XX, YY, zdata)
		
		ax2.set_title(plot_title)
		ax2.set_xlabel(x_title)
		ax2.set_ylabel(y_title)
		ax2.set_zlabel(z_title)
		ax2.set_yticks(ylabel)
		
		
		
	elif plotmode==1:
		ax1 = fig.add_subplot(1, 1, 1, projection='3d')
		surf = ax1.plot_surface(XX, YY, zdata, cmap=cmap_str, norm=Normalize(vmin=min_value, vmax=max_value))
		cb = fig.colorbar(surf, ax=ax1)
		
		ax1.set_title(plot_title)
		ax1.set_xlabel(x_title)
		ax1.set_ylabel(y_title)
		ax1.set_zlabel(z_title)
		
		ax1.set_yticks(ylabel)
		
	elif plotmode==2:
		
		ax1 = fig.add_subplot(1, 1, 1, projection='3d')
		
		ax1.plot_wireframe(XX, YY, zdata)
		# ax1.set_ylim(ydata.max(), ydata.min())
		ax1.set_title(plot_title)
		ax1.set_xlabel(x_title)
		ax1.set_ylabel(y_title)
		ax1.set_zlabel(z_title)
		
		ax1.set_yticks(ylabel)
		
	
	plt.tight_layout()
	
	if plotmode==3:
		fig.subplots_adjust(right=0.95, wspace=0.1)
	elif plotmode==1:
		fig.subplots_adjust(right=0.85)
	elif plotmode==2:
		fig.subplots_adjust(right=0.95)
	
	
	if len(imagepath) > 0:
		plt.savefig(imagepath)
	
	plt.show()
	

def Plot(plotmode, imagepath, cmap_str):
	global xdata
	global ydata
	global zdata
	global count
	global window_title
	global plot_title
	global x_title
	global y_title
	global z_title
	global min_value
	global max_value

	
	if ydata.size == 1:
		PlotOne(imagepath, cmap_str)
	else:
		Plot3D(plotmode, imagepath, cmap_str)



def SurfacePlot(min,\
				 max,\
				 plotmode,\
				 windowtitle,\
				 plottitle,\
				 imagepath,\
				 xtitle,\
				 ytitle,\
				 ztitle):

	global xdata
	global ydata
	global zdata
	global count
	global window_title
	global plot_title
	global x_title
	global y_title
	global z_title
	global min_value
	global max_value


	count = 0
	
	min_value = min
	max_value = max
	
	window_title = windowtitle
	plot_title = plottitle
	plot_mode = plotmode

	x_title = xtitle
	y_title = ytitle
	z_title = ztitle

	zdata = zdata.reshape((ydata.size, xdata.size))
	
	Plot(plotmode, imagepath, 'hot')





